from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.metadata.audio_file_metadata import AudioFileMetadata
from media_platform.service.media_platform_request import MediaPlatformRequest


class ExtractMetadataRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(ExtractMetadataRequest, self).__init__(authenticated_http_client, 'GET', base_url + '/audio/metadata', AudioFileMetadata)

        self.path = None

    def set_path(self, path):
        # type: (str) -> ExtractMetadataRequest
        self.path = path
        return self

    def execute(self):
        # type: () -> AudioFileMetadata
        return super(ExtractMetadataRequest, self).execute()

    def _params(self):
        # type: () -> dict
        return {
            'path': self.path
        }



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient


class MediaPlatformService(object):
    def __init__(self, domain, authenticated_http_client):
        # type: (str, AuthenticatedHTTPClient) -> None

        self._domain = domain

        self._authenticated_http_client = authenticated_http_client

        self._base_url = 'https://' + domain + '/_api'



from media_platform.job.convert_font_job import ConvertFontJob
from media_platform.job.create_archive_job import CreateArchiveJob
from media_platform.job.extract_archive_job import ExtractArchiveJob
from media_platform.job.extract_poster_job import ExtractPosterJob
from media_platform.job.extract_storyboard_job import ExtractStoryboardJob
from media_platform.job.import_file_job import ImportFileJob
from media_platform.job.job import Job
from media_platform.job.replace_extra_metadata_job import ReplaceExtraMetadataJob
from media_platform.job.subset_font_job import SubsetFontJob
from media_platform.job.transcode_job import TranscodeJob
from media_platform.lang.serialization import Deserializable


class _JobDeserializer(Deserializable):
    _job_classes = [
        ImportFileJob,
        ExtractPosterJob,
        ExtractStoryboardJob,
        CreateArchiveJob,
        ExtractArchiveJob,
        TranscodeJob,
        ReplaceExtraMetadataJob,
        ConvertFontJob,
        SubsetFontJob
    ]

    _type_to_class = {c.type: c for c in _job_classes}

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Job
        job_type = data['type']
        job_class = cls._type_to_class.get(job_type, Job)
        return job_class.deserialize(data)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.image_operation_job import ImageOperationSpecification
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class ImageOperationRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(ImageOperationRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/images/operations',
                                                    FileDescriptor)

        self.source = None
        self.specification = None

    def set_source(self, source):
        # type: (Source) -> ImageOperationRequest
        self.source = source
        return self

    def set_specification(self, specification):
        # type: (ImageOperationSpecification) -> ImageOperationRequest
        self.specification = specification
        return self

    def _params(self):
        return {
            'source': self.source.serialize(),
            'specification': self.specification.serialize(),
        }

    def execute(self):
        # type: () -> FileDescriptor
        return super(ImageOperationRequest, self).execute()



from media_platform.job.job import Job
from media_platform.lang.serialization import Deserializable


class JobCallbackPayload(Deserializable):
    def __init__(self, job, attachment=None):
        # type: (Job, dict or None) -> None
        self.job = job
        self.attachment = attachment

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> JobCallbackPayload
        return cls(Job.deserialize(data['job']), data.get('attachment'))



class AudioQuality(object):
    mp3_128 = 'mp3_128'
    mp3_256 = 'mp3_256'
    mp3_320 = 'mp3_320'
    aac_128 = 'aac_128'
    aac_196 = 'aac_196'
    aac_256 = 'aac_256'
    aac_320 = 'aac_320'
    flac = 'flac'
    alac = 'alac'

    values = [
        mp3_128,
        mp3_256,
        mp3_320,
        aac_128,
        aac_256,
        aac_320,
        flac,
        alac,
    ]

    @classmethod
    def has_value(cls, value):
        # type: (str) -> bool
        return value in cls.values



from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.flow_control_service.component import Component


class Flow(Serializable, Deserializable):
    def __init__(self, components=None):
        # type: (dict[str, Component]) -> None

        self.components = components or dict()

    def add_component(self, key, component):
        # type: (str, Component) -> Flow
        self.components[key] = component
        return self

    def validate(self):
        self._validate_successors()
        self._validate_acyclic()

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Flow
        components = {k: Component.deserialize(v) for k, v in data.items()}

        return Flow(components)

    def serialize(self):
        # type: () -> dict
        return {k: v.serialize() for k, v in self.components.items()}

    def _validate_successors(self):
        missing_successors = [s for c in self.components.values() for s in c.successors if s not in self.components]
        if missing_successors:
            raise ValueError('Missing successor components: %s' % missing_successors)

    def _validate_acyclic(self):
        graph = self.components
        path = set()
        visited = set()

        def visit(key):
            if key in visited:
                return False

            visited.add(key)
            path.add(key)

            if graph.get(key):
                successors = graph.get(key).successors
                for successor in successors:
                    if successor in path or visit(successor):
                        return True

            path.remove(key)
            return False

        if any(visit(v) for v in graph):
            raise ValueError('cyclic dependency detected')






from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.flow_control_service.flow_state import FlowState
from media_platform.service.media_platform_request import MediaPlatformRequest


class FlowStateRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(FlowStateRequest, self).__init__(authenticated_http_client, 'GET', base_url + '/flow_control/flow/',
                                               FlowState)

        self.id = None

        self._url = base_url + '/flow_control/flow/'

    def set_id(self, flow_id):
        # type: (str) -> FlowStateRequest
        self.id = flow_id
        return self

    def execute(self):
        # type: () -> FlowState
        self.url = self._url + self.id

        return super(FlowStateRequest, self).execute()



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.extract_archive_job import ExtractArchiveJob, ExtractionReport
from media_platform.service.callback import Callback
from media_platform.service.destination import Destination
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class ExtractArchiveRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(ExtractArchiveRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/archive/extract',
                                                    ExtractArchiveJob)

        self.source = None
        self.destination = None
        self.report = None
        self.callback = None

    def set_source(self, source):
        # type: (Source) -> ExtractArchiveRequest
        self.source = source
        return self

    def set_destination(self, destination):
        # type: (Destination) -> ExtractArchiveRequest
        self.destination = destination
        return self

    def set_report(self, extraction_report):
        # type: (ExtractionReport) -> ExtractArchiveRequest
        self.report = extraction_report
        return self

    def set_callback(self, callback):
        # type: (Callback) -> ExtractArchiveRequest
        self.callback = callback
        return self

    def execute(self):
        # type: () -> ExtractArchiveJob
        return super(ExtractArchiveRequest, self).execute()

    def _params(self):
        # type: () -> dict
        return {
            'source': self.source.serialize(),
            'destination': self.destination.serialize(),
            'extractedFilesReport': self.report.serialize() if self.report else None,
            'jobCallback': self.callback.serialize() if self.callback else None
        }



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.metadata.image.image_features import ImageFeatures
from media_platform.service.media_platform_request import MediaPlatformRequest


class Feature(object):
    faces = 'faces'
    labels = 'labels'
    colors = 'colors'
    explicit_content = 'explicit_content'

    values = [faces, labels, colors, explicit_content]

    @classmethod
    def has_value(cls, value):
        # type: (str) -> bool
        return value in cls.values


class ExtractFeaturesRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(ExtractFeaturesRequest, self).__init__(authenticated_http_client, 'GET', base_url + '/images/features',
                                                     ImageFeatures)

        self.path = None
        self.features = []

    def set_path(self, path):
        # type: (str) -> ExtractFeaturesRequest
        self.path = path
        return self

    def set_features(self, features):
        # type: ([Feature]) -> ExtractFeaturesRequest
        self.features = features
        return self

    def add_features(self, *features):
        # type: ([Feature]) -> ExtractFeaturesRequest
        self.features.extend(features)
        return self

    def _params(self):
        # type: () -> dict
        return {
            'path': self.path,
            'features': ','.join(self.features)
        }

    def execute(self):
        # type: () -> ImageFeatures
        return super(ExtractFeaturesRequest, self).execute()



import time
import requests
from requests import Response

from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.auth.token import Token
from media_platform.service.file_service.attachment import Attachment


class DownloadFileRequest(object):
    def __init__(self, app_id, authenticator, base_url):
        # type: (str, AppAuthenticator, str) -> None

        self.path = None
        self.ttl = 600  # seconds
        self.attachment = None
        self.on_expired_redirect_to = None

        self._app_urn = 'urn:app:' + app_id
        self._url = base_url + '/download/file'
        self._authenticator = authenticator

    def set_path(self, path):
        # type: (str) -> DownloadFileRequest
        self.path = path
        return self

    def set_ttl(self, ttl):
        # type: (int) -> DownloadFileRequest
        self.ttl = ttl
        return self

    def set_attachment(self, attachment):
        # type: (Attachment) -> DownloadFileRequest
        self.attachment = attachment
        return self

    def set_on_expired_redirect_to(self, on_expired_redirect_to):
        # type: (str) -> DownloadFileRequest
        self.on_expired_redirect_to = on_expired_redirect_to
        return self

    def url(self):
        # type: () -> str

        payload = {'path': self.path}
        if self.on_expired_redirect_to:
            payload['onExpireRedirectTo'] = self.on_expired_redirect_to
        if self.attachment:
            payload['attachment'] = self.attachment.serialize()

        token = Token(
            self._app_urn,
            self._app_urn,
            ['urn:service:file.download'],
            int(time.time()) - 10,
            int(time.time()) + self.ttl,
            {'payload': payload}
        )

        signed_token = self._authenticator.sign_token(token)

        return self._url + '?downloadToken=' + signed_token

    def execute(self):
        # type: () -> Response
        # http://docs.python-requests.org/en/master/user/advanced/#body-content-workflow

        # if you don't close the response, don't come complaining about connection leakage :)
        return requests.get(self._url, stream=True)



# This package is for specifications of flow components that are used ONLY as flow component specifications. If there
# is a job specification relating to a certain flow component, e.g. ImportFileSpecification, that is used as the
# component specification as well, and there will not be a separate specification class here.



from datetime import datetime

from media_platform.lang import datetime_serialization
from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.lifecycle import Lifecycle


class ACL(object):
    public = 'public'
    private = 'private'

    @classmethod
    def has_value(cls, value):
        return value in [cls.public, cls.private]


class FileType(object):
    file = '-'
    directory = 'd'
    symlink = 'l'


class FileMimeType(object):
    directory = 'application/vnd.wix-media.dir'
    symlink = 'application/vnd.wix-media.symlink'

    defualt = 'application/octet-stream'


class FileDescriptor(Serializable, Deserializable):
    def __init__(self, path, file_id, file_type, mime_type, size, acl=ACL.public, lifecycle=None, file_hash=None,
                 date_created=None, date_updated=None):
        # type: (str, str, str, str, int, ACL, Lifecycle, str, datetime, datetime) -> None
        super(FileDescriptor, self).__init__()

        self._validate_values(acl, path)

        self.path = path
        self.id = file_id
        self.type = file_type
        self.mime_type = mime_type
        self.size = size
        self.acl = acl
        self.lifecycle = lifecycle
        self.hash = file_hash
        self.date_created = date_created or datetime.utcnow()
        self.date_updated = date_updated or datetime.utcnow()

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> FileDescriptor

        lifecycle_data = data.get('lifecycle')
        lifecycle = Lifecycle.deserialize(lifecycle_data) if lifecycle_data else None

        return FileDescriptor(data['path'], data['id'], data['type'], data['mimeType'], data['size'], data['acl'],
                              lifecycle, data.get('hash'),
                              datetime_serialization.deserialize(data.get('dateCreated')),
                              datetime_serialization.deserialize(data.get('dateUpdated')))

    def serialize(self):
        # type: () -> dict
        return {
            'id': self.id,
            'path': self.path,
            'type': self.type,
            'mimeType': self.mime_type,
            'size': self.size,
            'hash': self.hash,
            'acl': self.acl,
            'lifecycle': self.lifecycle.serialize() if self.lifecycle else None,
            'dateUpdated': datetime_serialization.serialize(self.date_updated),
            'dateCreated': datetime_serialization.serialize(self.date_created)
        }

    @staticmethod
    def _validate_values(acl, path):
        FileDescriptor.path_validator(path)
        FileDescriptor.acl_validator(acl)

    @staticmethod
    def path_validator(path):
        # type: (str) -> None
        if not path.startswith('/'):
            raise ValueError('path must start with "/"' % path)

    @staticmethod
    def acl_validator(acl):
        if not ACL.has_value(acl):
            raise ValueError('ACL %s not supported' % acl)



from media_platform.lang.serialization import Deserializable


class ExplicitContent(Deserializable):
    def __init__(self, name, likelihood):
        # type: (str, str) -> None
        super(ExplicitContent, self).__init__()
        self.name = name
        self.likelihood = likelihood

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ExplicitContent
        return ExplicitContent(data['name'], data['likelihood'])



from media_platform.job.job import Job
from media_platform.job.specification import Specification
from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.destination import Destination
from media_platform.service.source import Source


class ExtractionReportFormat(object):
    csv = 'csv'
    json = 'json'

    @classmethod
    def has_value(cls, value):
        return value in [cls.csv, cls.json]


class ExtractionReport(Serializable, Deserializable):
    def __init__(self, destination, report_format=ExtractionReportFormat.csv):
        # type: (Destination, ExtractionReportFormat) -> None
        super(ExtractionReport, self).__init__()

        self.destination = destination
        self.format = report_format

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ExtractionReport
        destination = Destination.deserialize(data['destination'])

        return ExtractionReport(destination, data['format'])

    def serialize(self):
        # type: () -> dict
        return {
            'destination': self.destination.serialize(),
            'format': self.format
        }


class ExtractArchiveSpecification(Specification):
    def __init__(self, source, destination, extraction_report=None):
        # type: (Source, Destination, ExtractionReport or None) -> None

        self.source = source
        self.destination = destination
        self.extraction_report = extraction_report

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ExtractArchiveSpecification

        extraction_report_data = data.get('extractedFilesReport')
        if extraction_report_data:
            extraction_report = ExtractionReport.deserialize(extraction_report_data)
        else:
            extraction_report = None

        source_data = data.get('source')
        source = Source.deserialize(source_data) if source_data else None
        return ExtractArchiveSpecification(source,
                                           Destination.deserialize(data['destination']),
                                           extraction_report)

    def serialize(self):
        # type: () -> dict
        return {
            'source': self.source.serialize() if self.source else None,
            'destination': self.destination.serialize(),
            'extractedFilesReport': self.extraction_report.serialize() if self.extraction_report else None
        }


class ExtractArchiveJob(Job):
    type = 'urn:job:archive.extract'
    specification_type = ExtractArchiveSpecification



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.flow_control_service.abort_flow_request import AbortFlowRequest
from media_platform.service.flow_control_service.flow_invocation_request import FlowInvocationRequest
from media_platform.service.flow_control_service.flow_state_request import FlowStateRequest
from media_platform.service.media_platform_service import MediaPlatformService


class FlowControlService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client):
        # type: (str, AuthenticatedHTTPClient) -> None
        super(FlowControlService, self).__init__(domain, authenticated_http_client)

    def invoke_flow_request(self):
        # type: () -> FlowInvocationRequest
        return FlowInvocationRequest(self._authenticated_http_client, self._base_url)

    def flow_state_request(self):
        # type: () -> FlowStateRequest
        return FlowStateRequest(self._authenticated_http_client, self._base_url)

    def abort_flow_request(self):
        # type: () -> AbortFlowRequest
        return AbortFlowRequest(self._authenticated_http_client, self._base_url)



from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.file_descriptor import FileDescriptor


class Source(Serializable, Deserializable):
    def __init__(self, path=None, file_id=None):
        # type: (str, str) -> None

        self._validate(path, file_id)

        self.path = path
        self.file_id = file_id

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Source

        return Source(data.get('path'), data.get('fileId'))

    def serialize(self):
        # type: () -> dict
        return {
            'path': self.path,
            'fileId': self.file_id
        }

    @staticmethod
    def _validate(path, file_id):
        # type: (str, str) -> None

        if not (path or file_id):
            raise ValueError('path or file id must be specified')

        if path:
            FileDescriptor.path_validator(path)



from media_platform.lang.serialization import Deserializable


class AudioFormat(Deserializable):
    def __init__(self, long_name=None, bitrate=None, duration=None, size=None):
        # type: (str, int or None, int or None, int or None) -> None
        super(AudioFormat, self).__init__()
        self.long_name = long_name
        self.duration = duration
        self.bitrate = int(bitrate) if bitrate else None
        self.size = int(size) if size else None

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> AudioFormat
        return AudioFormat(data.get('formatLongName'),
                           data.get('bitrate'),
                           data.get('duration'),
                           data.get('size'))

    def serialize(self):
        # type: () -> dict
        return {
            'formatLongName': self.long_name,
            'duration': self.duration,
            'bitrate': self.bitrate,
            'size': self.size,
        }



from media_platform.lang.serialization import Serializable, Deserializable


class Lyrics(Serializable, Deserializable):
    def __init__(self, text, language=None, description=None):
        # type: (str, str, str) -> None
        self.description = description
        self.language = language
        self.text = text

        # backwards compatibility
        self.lang = self.language

    def serialize(self):
        # type: () -> dict
        return {
            'text': self.text,
            'description': self.description,
            'language': self.language,

            # backwards compatibility
            'lang': self.language,
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Lyrics
        return cls(data['text'], data.get('language') or data.get('lang'), data.get('description'))



from media_platform.job.specification import Specification
from media_platform.lang.serialization import Serializable
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.flow_control_service.component import Component, ComponentType
from media_platform.service.source import Source


class OperationStatus(object):
    waiting = 'waiting'
    aborted = 'aborted'
    working = 'working'
    success = 'success'
    error = 'error'


class Operation(Component):
    def __init__(self, component_type, specification, successors, status, delete_sources=False, sources=None,
                 results=None, jobs=None, extra_results=None, error_message=None, error_code=None):
        # type: (ComponentType, Specification, [str], OperationStatus, bool, [Source], [FileDescriptor] or [dict], [str], dict, str, int) -> None
        super(Operation, self).__init__(component_type, specification, successors, delete_sources)
        self.status = status
        self.sources = sources or []
        self.results = results or []
        self.jobs = jobs or []
        self.extra_results = extra_results or {}
        self.error_message = error_message
        self.error_code = error_code

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Operation
        component = super(Operation, cls).deserialize(data)

        sources_data = data.get('sources', [])
        results_data = data.get('results', [])

        sources = [Source.deserialize(s) for s in sources_data]
        results = [FileDescriptor.deserialize(r) for r in results_data]
        jobs = data.get('jobs', [])
        extra_results = data.get('extraResults', {})

        return Operation(component.type, component.specification, component.successors, data['status'],
                         component.delete_sources, sources, results, jobs, extra_results, data.get('errorMessage'),
                         data.get('errorCode'))

    def serialize(self):
        data = super(Operation, self).serialize()

        data.update({
            'sources': [s.serialize() for s in self.sources if s],
            'status': self.status,
            'results': [r.serialize() if isinstance(r, Serializable) else r for r in self.results],
            'jobs': self.jobs,
            'extraResults': self.extra_results
        })

        if self.error_code:
            data['errorCode'] = self.error_code

        if self.error_message:
            data['errorMessage'] = self.error_message

        return data



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_descriptor import FileType
from media_platform.service.file_service.file_list import FileList
from media_platform.service.list_request import _ListRequest


class FileListRequest(_ListRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(FileListRequest, self).__init__(authenticated_http_client, base_url + '/files/ls_dir', FileList)

        self.path = None
        self.recursive = False
        self.type = FileType.file

    def set_path(self, path):
        # type: (str) -> FileListRequest
        self.path = path
        return self

    def set_recursive(self, recursive):
        # type: (bool) -> FileListRequest
        self.recursive = recursive
        return self

    def set_type(self, file_type):
        # type: (str) -> FileListRequest
        self.type = file_type
        return self

    def set_next_page_token(self, next_page_token):
        # type: (str) -> FileListRequest
        return super(FileListRequest, self).set_next_page_token(next_page_token)

    def set_page_size(self, page_size):
        # type: (int) -> FileListRequest
        return super(FileListRequest, self).set_page_size(page_size)

    def set_order_by(self, order_by):
        # type: (str) -> FileListRequest
        return super(FileListRequest, self).set_order_by(order_by)

    def set_order_direction(self, order_direction):
        # type: (str) -> FileListRequest
        return super(FileListRequest, self).set_order_direction(order_direction)

    def execute(self):
        # type: () -> FileList
        return super(FileListRequest, self).execute()

    def _params(self):

        params = super(FileListRequest, self)._params()

        params['path'] = self.path

        if self.recursive:
            params['r'] = self.recursive

        if self.type:
            params['type'] = self.type

        return params



from media_platform.lang.serialization import Serializable, Deserializable


class Action(object):
    delete = 'delete'

    @classmethod
    def has_value(cls, value):
        return value in [cls.delete]


class Lifecycle(Serializable, Deserializable):
    def __init__(self, age, action):
        # type: (int, str) -> None
        super(Lifecycle, self).__init__()

        self._validate_values(action, age)

        self.age = age          # int seconds
        self.action = action    # delete

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Lifecycle

        return Lifecycle(data['age'], data['action'])

    def serialize(self):
        return {
            'age': self.age,
            'action': self.action
        }

    @staticmethod
    def _validate_values(action, age):
        if age < 30:
            raise ValueError('age must be greater than 30 seconds')

        if age > 365 * 24 * 60 * 60:
            raise ValueError('age must be less than 365 days')

        if not Action.has_value(action):
            raise ValueError('action %s not supported' % action)



import time

import requests
from requests import Response

from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.auth.token import Token


class ArchiveManifestUrlRequest(object):
    def __init__(self, app_id, authenticator, domain):
        # type: (str, AppAuthenticator, str) -> None
        self.path = None
        self.ttl = None  # seconds
        # self.attachment = None
        # self.on_expired_redirect_to = None

        self._app_urn = 'urn:app:' + app_id
        self._authenticator = authenticator

        self._url = '//archive-' + domain.replace('.appspot.com', '.wixmp.com')

    def set_path(self, path):
        # type: (str) -> ArchiveManifestUrlRequest
        self.path = path
        return self

    def set_ttl(self, ttl):
        # type: (int) -> ArchiveManifestUrlRequest
        self.ttl = ttl
        return self

    # todo
    # def set_attachment(self, attachment):
    #     # type: (Attachment) -> ArchiveManifestUrlRequest
    #     self.attachment = attachment
    #     return self
    #
    # def set_on_expired_redirect_to(self, on_expired_redirect_to):
    #     # type: (str) -> ArchiveManifestUrlRequest
    #     self.on_expired_redirect_to = on_expired_redirect_to
    #     return self

    def url(self):
        # type: () -> str
        url = self._url + self.path

        if self.ttl:
            auth_token = self._get_token()
            url += '?auth=' + auth_token

        return url

    def execute(self):
        # type: () -> Response
        # http://docs.python-requests.org/en/master/user/advanced/#body-content-workflow

        # if you don't close the response, don't come complaining about connection leakage :)
        return requests.get('https:' + self.url(), stream=True)

    def _get_token(self):
        # type: () -> str
        objects = [
            [{
                'path': self.path
            }]
        ]

        # payload = {}
        # if self.on_expired_redirect_to:
        #     payload['onExpireRedirectTo'] = self.on_expired_redirect_to
        # if self.attachment:
        #     payload['attachment'] = self.attachment.serialize()

        token = Token(
            self._app_urn,
            self._app_urn,
            ['urn:service:file.download'],
            int(time.time()) - 10,
            int(time.time()) + self.ttl,
            {'obj': objects}
        )

        return self._authenticator.sign_token(token)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.job import Job
from media_platform.service.media_platform_request import MediaPlatformRequest

# noinspection PyProtectedMember
from media_platform.job.job_deserializer import _JobDeserializer


class JobRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(JobRequest, self).__init__(authenticated_http_client, 'GET', base_url + '/jobs/', _JobDeserializer)

        self.id = None

        self._url = base_url + '/jobs/'

    def set_id(self, job_id):
        # type: (str) -> JobRequest
        self.id = job_id
        return self

    def execute(self):
        # type: () -> Job

        self.url = self._url + self.id

        return super(JobRequest, self).execute()



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_descriptor import FileDescriptor, FileMimeType, FileType, ACL
from media_platform.service.media_platform_request import MediaPlatformRequest


class CreateFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(CreateFileRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/files', FileDescriptor)

        self.path = None
        self.mime_type = FileMimeType.directory
        self.type = FileType.directory
        self.acl = ACL.public
        self.size = 0
        self.id = None
        self.bucket = None

    def set_path(self, path):
        # type: (str) -> CreateFileRequest
        self.path = path
        return self

    def set_mime_type(self, mime_type):
        # type: (FileMimeType) -> CreateFileRequest
        self.mime_type = mime_type
        return self

    def set_type(self, file_type):
        # type: (FileType) -> CreateFileRequest
        self.type = file_type
        return self

    def set_acl(self, acl):
        # type: (ACL) -> CreateFileRequest
        self.acl = acl
        return self

    def set_size(self, size):
        # type: (int) -> CreateFileRequest
        self.size = size
        return self

    def set_id(self, id):
        # type: (str) -> CreateFileRequest
        self.id = id
        return self

    def set_bucket(self, bucket):
        # type: (str) -> CreateFileRequest
        self.bucket = bucket
        return self

    def _params(self):
        return {
            'path': self.path,
            'mimeType': self.mime_type,
            'type': self.type,
            'acl': self.acl,
            'size': self.size,
            'id': self.id,
            'bucket': self.bucket
        }



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.import_file_job import ImportFileJob
from media_platform.lang.serialization import Serializable
from media_platform.service.destination import Destination
from media_platform.service.media_platform_request import MediaPlatformRequest


class ExternalAuthorization(Serializable):
    def __init__(self, headers):
        # type: (dict) -> None
        super(ExternalAuthorization, self).__init__()

        self.headers = headers

    def serialize(self):
        return {
            'headers': self.headers
        }


class ImportFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(ImportFileRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/import/file',
                                                ImportFileJob)

        self.source_url = None
        self.external_authorization = None  # type: ExternalAuthorization
        self.destination = None  # type: Destination

    def set_source_url(self, source_url):
        # type: (str) -> ImportFileRequest
        self.source_url = source_url
        return self

    def set_external_authorization(self, external_authorization):
        # type: (ExternalAuthorization) -> ImportFileRequest
        self.external_authorization = external_authorization
        return self

    def set_destination(self, destination):
        # type: (Destination) -> ImportFileRequest
        self.destination = destination
        return self

    def execute(self):
        # type: () -> ImportFileJob
        return super(ImportFileRequest, self).execute()

    def _params(self):
        return {
            'sourceUrl': self.source_url,
            'externalAuthorization': self.external_authorization.serialize() if self.external_authorization else None,
            'destination': self.destination.serialize()
        }



from datetime import datetime


ISO_DATE_WITH_MILLIS_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
ISO_DATE_FORMAT = '%Y-%m-%dT%H:%M:%SZ'


def deserialize(iso_date):
    # type: (str) -> datetime or None

    if iso_date is None:
        return None

    if '.' in iso_date:
        return datetime.strptime(iso_date, ISO_DATE_WITH_MILLIS_FORMAT)
    else:
        return datetime.strptime(iso_date, ISO_DATE_FORMAT)


def serialize(date_time=datetime.utcnow()):
    # type: (datetime) -> str

    return date_time.strftime(ISO_DATE_FORMAT)



from media_platform.job.job import Job
from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.service.source import Source


class ArchiveType(object):
    zip = 'zip'
    tar = 'tar'
    tar_gz = 'tar.gz'
    tar_bz2 = 'tar.bz2'

    @classmethod
    def has_value(cls, value):
        return value in [cls.zip, cls.tar, cls.tar_gz, cls.tar_bz2]


class ArchiveSource(Source):
    def __init__(self, path=None, file_id=None, path_in_archive=None):
        # type: (str, str, str) -> None
        super(ArchiveSource, self).__init__(path, file_id)

        self.path_in_archive = path_in_archive

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ArchiveSource
        return ArchiveSource(data.get('path'), data.get('fileId'), data.get('pathInArchive'))

    def serialize(self):
        # type: () -> dict
        data = super(ArchiveSource, self).serialize()
        data['pathInArchive'] = self.path_in_archive if self.path_in_archive else None

        return data

    @classmethod
    def from_source(cls, source):
        # type: (Source) -> ArchiveSource
        return ArchiveSource(source.path, source.file_id)


class CreateArchiveSpecification(Specification):
    def __init__(self, sources, destination, archive_type=ArchiveType.zip):
        # type: ([Source or ArchiveSource], Destination, ArchiveType) -> None

        self.sources = sources
        self.destination = destination
        self.archive_type = archive_type

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> CreateArchiveSpecification
        sources_data = data.get('sources')
        sources = [ArchiveSource.deserialize(source_info) for source_info in sources_data if source_info]

        return CreateArchiveSpecification(sources,
                                          Destination.deserialize(data['destination']),
                                          data['archiveType'])

    def serialize(self):
        # type: () -> dict
        return {
            'sources': [source.serialize() for source in self.sources if source],
            'destination': self.destination.serialize(),
            'archiveType': self.archive_type
        }


class CreateArchiveJob(Job):
    type = 'urn:job:archive.create'
    specification_type = CreateArchiveSpecification



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.callback import Callback
from media_platform.service.file_descriptor import ACL
from media_platform.service.file_service.upload_configuration import UploadConfiguration
from media_platform.service.media_platform_request import MediaPlatformRequest


class UploadConfigurationRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(UploadConfigurationRequest, self).__init__(authenticated_http_client, 'POST',
                                                         base_url + '/v2/upload/configuration', UploadConfiguration)
        self.path = None
        self.bucket = None
        self.mime_type = None
        self.acl = None
        self.size = None
        self.callback = None

    def set_path(self, path):
        # type: (str) -> UploadConfigurationRequest
        self.path = path
        return self

    def set_bucket(self, bucket):
        # type: (str) -> UploadConfigurationRequest
        self.bucket = bucket
        return self

    def set_mime_type(self, mime_type):
        # type: (str) -> UploadConfigurationRequest
        self.mime_type = mime_type
        return self

    def set_acl(self, acl):
        # type: (ACL) -> UploadConfigurationRequest
        self.acl = acl
        return self

    def set_size(self, size):
        # type: (int) -> UploadConfigurationRequest
        self.size = size
        return self

    def set_callback(self, callback):
        # type: (Callback) -> UploadConfigurationRequest
        self.callback = callback
        return self

    def execute(self):
        # type: () -> UploadConfiguration
        return super(UploadConfigurationRequest, self).execute()

    def validate(self):
        # todo
        super(UploadConfigurationRequest, self).validate()

    def _params(self):
        # type: () -> dict
        return {
            'path': self.path,
            'bucket': self.bucket,
            'mimeType': self.mime_type,
            'size': self.size,
            'acl': self.acl,
            'callback':  self.callback.serialize() if self.callback else None
        }



from media_platform.job.job import Job
from media_platform.job.specification import Specification
from media_platform.service.destination import Destination

# todo: proper class
SUPPORTED_IMAGE_FORMATS = ['jpg', 'png']


class ExtractStoryboardSpecification(Specification):
    def __init__(self, destination, columns, rows, tile_width=None, tile_height=None, image_format='jpg',
                 segment_duration=None):
        # type: (Destination, int, int, int, int, str, float) -> None
        super(ExtractStoryboardSpecification, self).__init__()

        self.destination = destination
        self.columns = columns
        self.rows = rows
        self.tile_width = tile_width
        self.tile_height = tile_height
        self.image_format = image_format
        self.segment_duration = segment_duration

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ExtractStoryboardSpecification
        destination = Destination.deserialize(data['destination'])

        return ExtractStoryboardSpecification(destination, data['columns'], data['rows'], data.get('tileWidth'),
                                              data.get('tileHeight'), data.get('format'), data.get('segmentDuration'))

    def serialize(self):
        # type: () -> dict
        return {
            'destination': self.destination.serialize(),
            'columns': self.columns,
            'rows': self.rows,
            'tileWidth': self.tile_width,
            'tileHeight': self.tile_height,
            'format': self.image_format,
            'segmentDuration': self.segment_duration
        }

    def validate(self):
        if self.image_format not in SUPPORTED_IMAGE_FORMATS:
            raise ValueError('image format must be one of: %s' % ', '.join(SUPPORTED_IMAGE_FORMATS))

        if self.image_format == 'jpg' and self.tile_width:
            self._validate_max_jpeg_size(self.columns, self.tile_width)

        if self.image_format == 'jpg' and self.tile_height:
            self._validate_max_jpeg_size(self.rows, self.tile_height)

    @staticmethod
    def _validate_max_jpeg_size(tiles, pixels):
        # type: (int, int) -> None
        if tiles * pixels > 65534:
            raise ValueError('jpeg supports up to 65k pixels')


class ExtractStoryboardJob(Job):
    type = 'urn:job:av.storyboard'
    specification_type = ExtractStoryboardSpecification



from media_platform.lang.serialization import Deserializable


class UploadConfiguration(Deserializable):
    def __init__(self, upload_url, upload_token):
        # type: (str, str) -> None
        super(UploadConfiguration, self).__init__()
        self.upload_url = upload_url
        self.upload_token = upload_token

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> UploadConfiguration
        return UploadConfiguration(data['uploadUrl'], data['uploadToken'])



import os

from media_platform.service.file_descriptor import FileDescriptor


class PlaylistRequest(object):
    # https://github.com/kaltura/nginx-vod-module#multi-url-structure
    URL_SET = '{url}{prefix},{renditions},{postfix}.urlset/master.m3u8'

    def __init__(self, domain):
        # type: (str) -> None

        self._url = '//packager-' + domain.replace('.appspot.com', '.wixmp.com')

        self.file_descriptors = []
        self.paths = []

    def set_files(self, file_descriptors):
        # type: ([FileDescriptor]) -> PlaylistRequest
        self.file_descriptors = file_descriptors
        return self

    def add_files(self, *file_descriptors):
        # type: ([FileDescriptor]) -> PlaylistRequest
        self.file_descriptors.extend(file_descriptors)
        return self

    def set_paths(self, paths):
        # type: ([str]) -> PlaylistRequest
        self.paths = paths
        return self

    def add_paths(self, *paths):
        # type: ([str]) -> PlaylistRequest
        self.paths.extend(paths)
        return self

    def execute(self):
        # type: () -> str
        paths = [file_descriptor.path for file_descriptor in self.file_descriptors]
        paths.extend(self.paths)

        common_prefix = self._common_prefix(paths)
        common_suffix = self._common_suffix(paths)

        renditions = [path.replace(common_prefix, '').replace(common_suffix, '') for path in paths]

        return self.URL_SET.format(url=self._url,
                                   prefix=common_prefix,
                                   renditions=','.join(renditions),
                                   postfix=common_suffix)

    @staticmethod
    def _common_prefix(paths):
        # type: ([str]) -> str
        return os.path.commonprefix(paths)

    @staticmethod
    def _common_suffix(paths):
        # type: ([str]) -> str
        reversed_paths = [path[::-1] for path in paths]

        reversed_common = os.path.commonprefix(reversed_paths)

        return reversed_common[::-1]



from media_platform.job.job import Job
from media_platform.job.job_deserializer import _JobDeserializer
from media_platform.lang.serialization import Deserializable


class _JobGroupResponse(Deserializable):
    def __init__(self, jobs):
        # type: ([Job]) -> None
        super(_JobGroupResponse, self).__init__()

        self.jobs = jobs

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> _JobGroupResponse
        return _JobGroupResponse(
            [_JobDeserializer.deserialize(item) for item in data]
        )



from media_platform.metadata.file_metadata import FileMetadata, MediaType
from media_platform.metadata.image.image_basic import ImageBasic
from media_platform.metadata.image.image_features import ImageFeatures
from media_platform.service.file_descriptor import FileDescriptor


class ImageFileMetadata(FileMetadata):
    def __init__(self, file_descriptor, basic=None, features=None):
        # type: (FileDescriptor, ImageBasic, ImageFeatures) -> None
        super(ImageFileMetadata, self).__init__(MediaType.image, file_descriptor, basic)

        self.features = features

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ImageFileMetadata
        if data['mediaType'] != MediaType.image:
            raise ValueError('not image metadata')

        file_descriptor = FileDescriptor.deserialize(data['fileDescriptor'])
        basic = ImageBasic.deserialize(data.get('basic')) if data.get('basic') else None
        features = ImageFeatures.deserialize(data.get('features')) if data.get('features') else None

        return ImageFileMetadata(file_descriptor, basic, features)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_service import MediaPlatformService
from media_platform.service.text_service.convert_font_request import ConvertFontRequest
from media_platform.service.text_service.subset_font_request import SubsetFontRequest


class TextService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client):
        # type: (str, AuthenticatedHTTPClient) -> None
        super(TextService, self).__init__(domain, authenticated_http_client)

    def convert_font_request(self):
        # type: () -> ConvertFontRequest
        return ConvertFontRequest(self._authenticated_http_client, self._base_url)

    def subset_font_request(self):
        # type: () -> SubsetFontRequest
        return SubsetFontRequest(self._authenticated_http_client, self._base_url)



from media_platform.lang.serialization import Serializable


class AttachedImage(Serializable):
    # http://id3.org/id3v2.4.0-frames (clause 4.14)

    def __init__(self, picture_type, mime_type=None, url=None):
        # type: (str, str, str) -> None
        super(AttachedImage, self).__init__()
        self.picture_type = picture_type
        self.mime_type = mime_type  # "image/png" or "image/jpeg"
        self.url = url  # mime tag will be  "-->" if url

    @classmethod
    def deserialize(cls, data):
        return AttachedImage(data.get('type'), data.get('mimeType'), data.get('url'))

    def serialize(self):
        return {
            'type': self.picture_type,
            'mimeType': self.mime_type,
            'url': self.url
        }


#    Picture type:  $00  Other
#                   $01  32x32 pixels 'file icon' (PNG only)
#                   $02  Other file icon
#                   $03  Cover (front)
#                   $04  Cover (back)
#                   $05  Leaflet page
#                   $06  Media (e.g. label side of CD)
#                   $07  Lead artist/lead performer/soloist
#                   $08  Artist/performer
#                   $09  Conductor
#                   $0A  Band/Orchestra
#                   $0B  Composer
#                   $0C  Lyricist/text writer
#                   $0D  Recording Location
#                   $0E  During recording
#                   $0F  During performance
#                   $10  Movie/video screen capture
#                   $11  A bright coloured fish
#                   $12  Illustration
#                   $13  Band/artist logotype
#                   $14  Publisher/Studio logotype



from media_platform.service.destination import Destination
from media_platform.job.specification import Specification


class CopyFileSpecification(Specification):
    def __init__(self, destination):
        # type: (Destination) -> None
        self.destination = destination

    def serialize(self):
        # type: () -> dict
        return {
            'destination': self.destination.serialize()
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> CopyFileSpecification
        return cls(Destination.deserialize(data['destination']))



from media_platform.lang.serialization import Serializable, Deserializable


class Callback(Serializable, Deserializable):
    def __init__(self, url, attachment=None, headers=None, passthrough=False):
        # type: (str, dict, dict, bool) -> None

        self.url = url
        self.attachment = attachment
        self.headers = headers
        self.passthrough = passthrough

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Callback
        return cls(data['url'], data.get('attachment'), data.get('headers'), data.get('passthrough', False))

    def serialize(self):
        # type: () -> dict
        return {
            'url': self.url,
            'attachment': self.attachment,
            'headers': self.headers,
            'passthrough': self.passthrough
        }



from setuptools import setup, find_packages

PACKAGE_NAME = 'wix_media_platform'
PACKAGE_VERSION = '1.12.0'

setup(
    name=PACKAGE_NAME,
    version=PACKAGE_VERSION,
    description='Wix Media Platform python SDK',
    author='Wix Media Platform',
    author_email='mcloud@wix.com',
    url='https://github.com/wix/media-platform-python-sdk',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    packages=find_packages(include=('media_platform', 'media_platform.*',)),
    install_requires=[
        'python-jose==3.0.1',
        'requests==2.20.1',
        'requests-toolbelt==0.8.0',
        'typing==3.6.6'
    ],
    tests_require=[
        'PyHamcrest==1.9.0',
        'mockito==1.1.1',
        'httpretty==0.9.6'
    ]
)



from media_platform.lang.serialization import Deserializable


class Label(Deserializable):
    def __init__(self, name, score):
        # type: (str, float) -> None
        super(Label, self).__init__()
        self.name = name
        self.score = score

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Label
        return Label(data['name'], data['score'])



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_service import MediaPlatformService
from media_platform.service.video_service.extract_poster_request import ExtractPosterRequest
from media_platform.service.video_service.extract_storyboard_request import ExtractStoryboardRequest


class VideoService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client):
        # type: (str, AuthenticatedHTTPClient) -> None
        super(VideoService, self).__init__(domain, authenticated_http_client)

    def extract_poster_request(self):
        # type: () -> ExtractPosterRequest
        return ExtractPosterRequest(self._authenticated_http_client, self._base_url)

    def extract_storyboard_request(self):
        # type: () -> ExtractStoryboardRequest
        return ExtractStoryboardRequest(self._authenticated_http_client, self._base_url)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.media_platform_request import MediaPlatformRequest


class DeleteFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(DeleteFileRequest, self).__init__(authenticated_http_client, 'DELETE', base_url + '/files', None)

        self.path = None

    def set_path(self, path):
        # type: (str) -> DeleteFileRequest
        self.path = path
        return self

    def execute(self):
        # type: () -> None
        return super(DeleteFileRequest, self).execute()

    def validate(self):
        FileDescriptor.path_validator(self.path)

    def _params(self):
        return {
            'path': self.path
        }



from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.job.job import Job

# todo: proper class
SUPPORTED_IMAGE_FORMATS = ['jpg', 'png']


class ExtractPosterSpecification(Specification):
    def __init__(self, second, destination, image_format='jpg', percentage=None):
        # type: (float or None, Destination, str, float or None) -> None
        super(ExtractPosterSpecification, self).__init__()

        self.second = second
        self.percentage = percentage
        self.destination = destination
        self.image_format = image_format or 'jpg'

    def serialize(self):
        # type: () -> dict
        return {
            'second': self.second,
            'percentage': self.percentage,
            'destination': self.destination.serialize(),
            'format': self.image_format
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ExtractPosterSpecification
        destination = Destination.deserialize(data['destination'])

        return ExtractPosterSpecification(data['second'], destination, data['format'])

    def validate(self):
        self._validate_image_format()
        self._validate_time()

    def _validate_image_format(self):
        if self.image_format not in SUPPORTED_IMAGE_FORMATS:
            raise ValueError('image format must be one of: %s' % ', '.join(SUPPORTED_IMAGE_FORMATS))

    def _validate_time(self):
        if self.second is not None:
            return

        if self.percentage is None:
            raise ValueError('must provide second or percentage')

        if self.percentage < 0 or self.percentage > 100:
            raise ValueError('percentage must be 0-100')


class ExtractPosterJob(Job):
    type = 'urn:job:av.poster'
    specification_type = ExtractPosterSpecification
